@@ -35,6 +35,7 @@ module Filesystem.Path
 	, commonPrefix
 	, stripPrefix
 	, collapse
+	, splitDirectories
 	
 	-- * Extensions
 	, extension
@@ -212,11 +213,11 @@ commonPrefix ps = foldr1 step ps where
 		then x : common xs ys
 		else []
 
--- Remove a prefix from a path.
+-- | Remove a prefix from a path.
 --
 -- @
--- stripPrefix "/foo/" "/foo/bar/baz.txt" == Just "bar/baz.txt"
--- stripPrefix "/foo/" "/bar/baz.txt" == Nothing
+-- 'stripPrefix' \"/foo/\" \"/foo/bar/baz.txt\" == Just \"bar/baz.txt\"
+-- 'stripPrefix' \"/foo/\" \"/bar/baz.txt\" == Nothing
 -- @
 --
 -- Since: 0.4.1
@@ -269,6 +270,23 @@ collapse p = p { pathDirectories = reverse newDirs } where
 		       | otherwise -> (False, ts)
 	step (_, acc) c = (False, c:acc)
 
+splitDirectories :: FilePath -> [FilePath]
+splitDirectories p = rootName ++ dirNames ++ fileName where
+	rootName = case pathRoot p of
+		Nothing -> []
+		r -> [asFile (rootChunk r)]
+	dirNames = map asFile (pathDirectories p)
+	fileName = case (pathBasename p, pathExtensions p) of
+		(Nothing, []) -> []
+		_ -> [filename p]
+	
+	asFile :: Chunk -> FilePath
+	asFile c = case parseFilename c of
+		(base, exts) -> empty
+			{ pathBasename = base
+			, pathExtensions = exts
+			}
+
 -------------------------------------------------------------------------------
 -- Extensions
 -------------------------------------------------------------------------------
